//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

using DesignSurfaceExtenderLibrary.Extensions;

namespace ConvexHullDesignSurfaceExtension
{   
    public class ConvexHullExtension : DesignSurfaceExtension
    {
        // IReferenceService workaround.
        private ConvexHullGlyph hullGlyph;

        public ConvexHullExtension( IServiceProvider serviceProvider )
            : base( serviceProvider )
        {

        }

        protected override void PopulateAdorner( Adorner adorner )
        {
            adorner.Glyphs.Clear();

            hullGlyph = new ConvexHullGlyph(
                this.BehaviorService,
                this.ComponentChangeService,
                this.SelectionService,
                this.ReferenceService,
                this.ServiceProvider,
                adorner );

            adorner.Glyphs.Add( hullGlyph );
        }

        protected override void AddGlyphsForControl( Control control )
        {
            // No implementation.
        }

        protected override void RemoveGlyphsForControl( Control control )
        {
            // IReferenceService workaround.
            if( this.hullGlyph != null )
            {
                this.hullGlyph.ComputeGeometry( control );
            }
        }

        public override DesignerActionList ActionList
        {
            get
            {
                if( this.actionListValue == null )
                {
                    this.actionListValue = new ConvexHullActionList( this );
                }

                return this.actionListValue;
            }
        }
    }
}
